require("matrix")
require("geometry")

function love.load()
    theta = 0 
    triangles = {
        --south
        Triangle.new({{0,0,0},{0,1,0},{1,1,0}}),
        Triangle.new({{0,0,0},{1,1,0},{1,0,0}}),
        --north
        Triangle.new({{1,0,1},{1,1,1},{0,1,1}}),
        Triangle.new({{1,0,1},{0,1,1},{0,0,1}}),
        --east
        Triangle.new({{1,0,0},{1,1,0},{1,1,1}}),
        Triangle.new({{1,0,0},{1,1,1},{1,0,1}}),
        --west
        Triangle.new({{0,0,1},{0,1,1},{0,1,0}}),
        Triangle.new({{0,0,1},{0,1,0},{0,0,0}}),
        --top
        Triangle.new({{0,1,0},{0,1,1},{1,1,1}}),
        Triangle.new({{0,1,0},{1,1,1},{1,1,0}}),
        --bottom
        Triangle.new({{1,0,1},{0,0,1},{0,0,0}}),
        Triangle.new({{1,0,1},{0,0,0},{1,0,0}})

    }

    cube = Mesh.new(triangles)

    --projection matrix
    near = 0.1
    far = 1000.0
    fov = 90.0
    AspectRatio = love.graphics.getHeight() / love.graphics.getWidth()
    fovRad = 1 / math.tan(fov * 0.5 / 180 * math.pi)
    --this matrix helps project our 2d coordinates into 3d
    ProjectionMatrix = Matrix.new(
        {
            {    AspectRatio * fovRad, 0, 0, 0},
            {0,     fovRad, 0,               0},
            {0,0,  far / (far - near),       1},
            {0,0,(-far * near)/(far - near) - 3, 0}
        })
    --this matrix helps rotate our object around the z axis(initially empty but we update values in the update function)
    matRotZ = Matrix.new(
        {
            {0,0,0,0},
            {0,0,0,0},
            {0,0,0,0},
            {0,0,0,0}
        })
    --this matrix helps rotate our object around the x axis(same deal with the rotation matrix for z)
    matRotX = Matrix.new(
        {
            {0,0,0,0},
            {0,0,0,0},
            {0,0,0,0},
            {0,0,0,0}
        })
    vCamera = vec3d.new(0,0,0)
end

function love.update(dt)
    --this is where we update our rotation matrices
    theta = theta + 1 * dt
    -- for the z component, we updates valiues of the rotation matrix
    matRotZ.field[1][1] = math.cos(theta)
    matRotZ.field[1][2] = math.sin(theta)
    matRotZ.field[2][1] = -math.sin(theta)
    matRotZ.field[2][2] = math.cos(theta)
    matRotZ.field[3][3] = 1
    matRotZ.field[4][4] = 1
    --for the x translation we rotate theta by .5 in order for the speeds to be consistant
    matRotX.field[1][1] = 1
    matRotX.field[2][2] = math.cos(theta * 0.5)
    matRotX.field[2][3] = math.sin(theta * 0.5)
    matRotX.field[3][2] = -math.sin(theta * 0.5)
    matRotX.field[3][3] = math.cos(theta * 0.5)
    matRotX.field[4][4] = 1

end

--this function creates a copy of our og matrix so that we don't update the value of the cube table
function deepcopy(orig)
    local copy = {}
    for k, v in pairs(orig) do
        if type(v) == "table" then
            copy[k] = deepcopy(v)
        else
            copy[k] = v
        end
    end
    return copy
end

--responcible for drawing the cube to the screen
function love.draw()
    --we iterate over every triangle that makes up the cube
    for i, face in ipairs(cube.triangles) do
        --three triangles involved in our major operations
        local trigon = Triangle.new({{0,0,0},{0,0,0},{0,0,0}})
        local triRotatedZ = Triangle.new({{0,0,0},{0,0,0},{0,0,0}})
        local triRotatedZX = Triangle.new({{0,0,0},{0,0,0},{0,0,0}})

        --first we rotate unscaled coordinates using the z rotation matrix
        Matrix:mult(deepcopy(face.vertices[1]), triRotatedZ.vertices[1], matRotZ.field)
        Matrix:mult(deepcopy(face.vertices[2]), triRotatedZ.vertices[2], matRotZ.field)
        Matrix:mult(deepcopy(face.vertices[3]), triRotatedZ.vertices[3], matRotZ.field)

        --second, we do the same but we rotate on the x plane
        Matrix:mult(triRotatedZ.vertices[1], triRotatedZX.vertices[1], matRotX.field)
        Matrix:mult(triRotatedZ.vertices[2], triRotatedZX.vertices[2], matRotX.field)
        Matrix:mult(triRotatedZ.vertices[3], triRotatedZX.vertices[3], matRotX.field)

        --we thn project translate the rotated vertices
        local triTranslated = triRotatedZX.vertices
        --inc z by 3
        triTranslated[1][3] = triRotatedZX.vertices[1][3] + 3
        triTranslated[2][3] = triRotatedZX.vertices[2][3] + 3
        triTranslated[3][3] = triRotatedZX.vertices[3][3] + 3
        --we calculate normals after we translate triangle to world space but before...
        --we actually project it
        --we calculate the vectors between 2 lines of the 
        local line1 = vec3d.new(triTranslated[2][1] - triTranslated[1][1], 
                          triTranslated[2][2] - triTranslated[1][2],
                          triTranslated[2][3] - triTranslated[1][3])
        local line2 = vec3d.new(triTranslated[3][1] - triTranslated[1][1], 
                          triTranslated[3][2] - triTranslated[1][2],
                          triTranslated[3][3] - triTranslated[1][3])
        --then we calculate the crossproduct and store it as the normal vector
        local normal = vec3d.new(line1.y * line2.z - line1.z * line2.y,
                            -(line1.x * line2.z - line1.z * line2.x),
                             line1.x * line2.y - line1.y * line2.x) 
        
        --Here we calculate the unit normal
        l = math.sqrt(normal.x * normal.x + normal.y * normal.y + normal.x * normal.z)
        normal.x = normal.x / l
        normal.y = normal.y / l
        normal.z = normal.z / l

        --we want to draw if the dot product betwen the normal and tri translates is < 0
        local dotprod = normal.x * (triTranslated[1][1] - vCamera.x) + 
                        normal.y * (triTranslated[1][2] - vCamera.x) + 
                        normal.z * (triTranslated[1][3] - vCamera.x)
        if dotprod < 0 then
            --project into 3d
            Matrix:mult(triTranslated[1], trigon.vertices[1], ProjectionMatrix.field)
            Matrix:mult(triTranslated[2], trigon.vertices[2], ProjectionMatrix.field)
            Matrix:mult(triTranslated[3], trigon.vertices[3], ProjectionMatrix.field)
            --xy translation
            trigon.vertices[1][1] = trigon.vertices[1][1] + 1
            trigon.vertices[1][2] = trigon.vertices[1][2] + 1
            trigon.vertices[2][1] = trigon.vertices[2][1] + 1
            trigon.vertices[2][2] = trigon.vertices[2][2] + 1
            trigon.vertices[3][1] = trigon.vertices[3][1] + 1
            trigon.vertices[3][2] = trigon.vertices[3][2] + 1
            -- * 0.5 * love.graphics.getWidth()
            trigon.vertices[1][1] = trigon.vertices[1][1] * 0.5 * love.graphics.getWidth()
            trigon.vertices[1][2] = trigon.vertices[1][2] * 0.5 * love.graphics.getWidth()
            trigon.vertices[2][1] = trigon.vertices[2][1] * 0.5 * love.graphics.getWidth()
            trigon.vertices[2][2] = trigon.vertices[2][2] * 0.5 * love.graphics.getWidth()
            trigon.vertices[3][1] = trigon.vertices[3][1] * 0.5 * love.graphics.getWidth()
            trigon.vertices[3][2] = trigon.vertices[3][2] * 0.5 * love.graphics.getWidth()
            --drawing the triangles
            love.graphics.polygon("line", (trigon.vertices[1][1]), (trigon.vertices[1][2]),
                                        (trigon.vertices[2][1]), (trigon.vertices[2][2]),
                                        (trigon.vertices[3][1]), (trigon.vertices[3][2]))
        end
    
    end    
end